'use strict';

function _interopDefault (ex) { return 'default' in ex ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var uniqueId = _interopDefault(require('mini-unique-id'));
var Waiting = _interopDefault(require('waiting'));
var browserVendorPrefix = require('browser-vendor-prefix');
var toCSS = _interopDefault(require('style-to-css'));
var panelsUi = require('panels-ui');
var stringify = _interopDefault(require('json-stable-stringify'));

var babelHelpers = {};
babelHelpers.typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj;
};

babelHelpers.classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

babelHelpers.createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

babelHelpers.extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

babelHelpers.inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

babelHelpers.objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

babelHelpers.possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

babelHelpers.slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

babelHelpers;

var Code = function (_Component) {
  babelHelpers.inherits(Code, _Component);

  function Code(props) {
    babelHelpers.classCallCheck(this, Code);

    var _this = babelHelpers.possibleConstructorReturn(this, Object.getPrototypeOf(Code).call(this, props));

    _this.id = 'code-' + uniqueId();
    return _this;
  }

  babelHelpers.createClass(Code, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      var props = this.props;

      var editor = ace.edit(this.id);
      this.editor = editor;

      var session = editor.getSession();

      session.setMode('ace/mode/' + props.mode);
      session.setTabSize(2);
      session.setUseSoftTabs(true);
      session.setUseWrapMode(true);

      editor.setDisplayIndentGuides(false);
      editor.setFontSize(12);
      editor.setHighlightActiveLine(props.highlightActiveLine);
      editor.setTheme('ace/theme/' + props.theme);
      editor.setOption('readOnly', props.readOnly);
      editor.setOption('wrap', props.wrap);
      editor.renderer.setShowGutter(props.gutter);

      if (props.src) {
        fetch(props.src).then(function (res) {
          return res.text();
        }).then(function (code) {
          return _this2.editor.setValue(code, -1);
        });
      } else if (props.code) {
        editor.setValue(props.code, -1);
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps) {
      var editor = this.editor;
      var props = this.props;

      var session = editor.getSession();

      if (prevProps.mode !== props.mode) {
        session.setMode('ace/mode/' + props.mode);
      }
      if (prevProps.code !== props.code) {
        editor.setValue(props.code, -1);
      }
      if (prevProps.gutter !== props.gutter) {
        editor.renderer.setShowGutter(props.gutter);
      }
      if (prevProps.highlightActiveLine !== props.highlightActiveLine) {
        editor.setHighlightActiveLine(props.highlightActiveLine);
      }

      if (prevProps.readyOnly !== props.readOnly) {
        editor.setOption('readOnly', props.readOnly);
      }
      if (prevProps.src !== props.src) {
        fetch(props.src).then(function (res) {
          return res.text();
        }).then(function (code) {
          return editor.setValue(code, -1);
        });
      }
      if (prevProps.wrap !== props.wrap) {
        editor.setOption('wrap', props.wrap);
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var id = this.id;


      return React__default.createElement(
        'div',
        { id: this.props.id },
        React__default.createElement(
          'style',
          null,
          '\n          #' + id + ' span {\n            display: inline;\n          }\n          #' + id + ' div {\n            display: block;\n          }\n          #' + id + ' .ace_gutter-cell span {\n            display: inline-block;\n          }\n        '
        ),
        React__default.createElement('div', { id: id, style: babelHelpers.extends({}, style, this.props.style) })
      );
    }
  }]);
  return Code;
}(React.Component);

Code.defaultProps = {
  gutter: true,
  highlightActiveLine: true,
  mode: 'json',
  theme: 'idle_fingers',
  wrap: 40
};

var style = {
  height: 250,
  width: '100%'
};

var OverCover = function OverCover(props) {
  var overCoverStyle = babelHelpers.extends({}, style$1.overCover, {
    height: props.height,
    left: 'calc(50% - ' + props.width / 2 + 'px)',
    top: 'calc(50% - ' + props.height / 2 + 'px)',
    width: props.width
  });

  return React__default.createElement('img', { src: props.src, style: props.style });
};

var Embed = function (_Component) {
  babelHelpers.inherits(Embed, _Component);

  function Embed(props) {
    babelHelpers.classCallCheck(this, Embed);

    var _this = babelHelpers.possibleConstructorReturn(this, Object.getPrototypeOf(Embed).call(this, props));

    _this.state = {
      error: !!props.src === false,
      isLoading: !!props.src,
      isReady: false
    };
    return _this;
  }

  babelHelpers.createClass(Embed, [{
    key: 'componentWillMount',
    value: function componentWillMount() {
      this.getEmbed(this.props.src);
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      if (this.props.src !== nextProps.src) {
        this.getEmbed(nextProps.src);
      }
    }
  }, {
    key: 'getEmbed',
    value: function getEmbed(src) {
      var _this2 = this;

      fetch(src).then(function (res) {
        return res.json();
      }).then(function (data) {
        _this2.setState({
          data: data,
          isLoading: false,
          isReady: true,
          showCover: !!data.thumbnail_url
        });
      }).catch(function (err) {
        _this2.setState({
          error: true,
          message: err
        });
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _this3 = this;

      var _state = this.state;
      var data = _state.data;
      var error = _state.error;
      var isLoading = _state.isLoading;
      var isReady = _state.isReady;
      var showCover = _state.showCover;
      var _props = this.props;
      var height = _props.height;
      var overCover = _props.overCover;
      var width = _props.width;

      var embedStyle = babelHelpers.extends({
        height: height,
        width: width
      }, this.props.style);
      var ret = void 0;

      if (isLoading) {
        ret = React__default.createElement(Waiting, null);
      } else if (isReady) {
        if (showCover) {
          ret = React__default.createElement(
            'div',
            { onClick: function onClick() {
                return _this3.setState({ showCover: false });
              }, style: style$1.cover },
            overCover && React__default.createElement(OverCover, overCover),
            React__default.createElement('img', { src: data.thumbnail_url, style: style$1.coverImage })
          );
        } else {
          ret = React__default.createElement('div', { dangerouslySetInnerHTML: { __html: data.html } });
        }
      } else if (error) {
        ret = React__default.createElement(
          'div',
          null,
          typeof message === 'undefined' ? 'Do you src in your props?' : message
        );
      }

      return React__default.createElement(
        'div',
        { style: embedStyle },
        ret
      );
    }
  }]);
  return Embed;
}(React.Component);

Embed.defaultProps = {
  overCover: false
};

var style$1 = {
  cover: {
    cursor: 'pointer',
    position: 'relative'
  },
  overCover: {
    position: 'absolute',
    zIndex: 1
  }
};

var GoTo = function GoTo(props) {
  var styleHover = props.styleHover;
  var rest = babelHelpers.objectWithoutProperties(props, ['styleHover']);

  var className = 'GoTo-' + uniqueId();

  var inlineStyle = props.styleHover ? '.' + className + ':hover {' + toCSS(styleHover) + '}' : '';

  return React__default.createElement(
    'a',
    babelHelpers.extends({}, rest, { className: className, target: '_blank' }),
    props.children,
    React__default.createElement(
      'style',
      null,
      inlineStyle
    )
  );
};

var clipboardText = null;

document.addEventListener('copy', function (ev) {
  if (clipboardText) {
    ev.preventDefault();
    ev.clipboardData.setData('text/plain', clipboardText);
    clipboardText = null;
  }
});

function copyToClipboard(text) {
  clipboardText = text;
  // debugger
  document.execCommand('copy');
}

var RESET_ACTIVE_TIMEOUT = 1000;

var OnClick = function (_Component) {
  babelHelpers.inherits(OnClick, _Component);

  function OnClick(props) {
    babelHelpers.classCallCheck(this, OnClick);

    var _this = babelHelpers.possibleConstructorReturn(this, Object.getPrototypeOf(OnClick).call(this, props));

    _this.state = {
      active: false
    };
    return _this;
  }

  babelHelpers.createClass(OnClick, [{
    key: 'bindOnClick',
    value: function bindOnClick(onClick) {
      var _this2 = this;

      var finalOnClick = void 0;

      if (/^copy:/.test(onClick)) {
        (function () {
          var from = onClick.match(/^copy:(.+)/)[1];

          finalOnClick = function finalOnClick() {
            var $el = document.querySelector(from);
            var $editor = Array.from($el.childNodes).find(function ($c) {
              return $c.classList.contains('ace_editor');
            });
            var text = $editor ? ace.edit($editor.id).getValue() : $el.innerText;

            copyToClipboard(text);
          };
        })();
      } else if (typeof onClick === 'function') {
        finalOnClick = onClick;
      } else {
        finalOnClick = function finalOnClick() {
          return console.log(onClick);
        };
      }

      this.onClick = function (event) {
        finalOnClick(event);

        _this2.setState({
          active: true
        });

        _this2.onClickTimeout = setTimeout(function () {
          _this2.setState({
            active: false
          });
          _this2.onClickTimeout = null;
        }, RESET_ACTIVE_TIMEOUT);
      };
    }
  }, {
    key: 'componentWillMount',
    value: function componentWillMount() {
      this.bindOnClick(this.props.onClick);
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (this.onClickTimeout) {
        clearTimeout(this.onClickTimeout);
      }
    }
  }, {
    key: 'componentWillUpdate',
    value: function componentWillUpdate(nextProps) {
      this.bindOnClick(nextProps.onClick);
    }
  }, {
    key: 'render',
    value: function render() {
      var _this3 = this;

      var _props = this.props;
      var children = _props.children;
      var onClick = _props.onClick;
      var style = _props.style;
      var styleActive = _props.styleActive;
      var styleHover = _props.styleHover;
      var rest = babelHelpers.objectWithoutProperties(_props, ['children', 'onClick', 'style', 'styleActive', 'styleHover']);
      var active = this.state.active;

      var className = 'OnClick-' + uniqueId();

      var inlineStyle = !active && styleHover ? '.' + className + ':hover {' + toCSS(styleHover) + '}' : '';

      var finalStyle = active ? babelHelpers.extends({}, style, styleActive, {
        outline: 0
      }) : babelHelpers.extends({}, style, {
        outline: 0,
        cursor: 'pointer'
      });

      return React__default.createElement(
        'button',
        babelHelpers.extends({}, rest, { className: className, disabled: active, onClick: function onClick() {
            return _this3.onClick();
          }, style: finalStyle }),
        children,
        React__default.createElement(
          'style',
          null,
          inlineStyle
        )
      );
    }
  }]);
  return OnClick;
}(React.Component);

function createGroup(name, style) {
  var Group = function (_Component) {
    babelHelpers.inherits(Group, _Component);

    function Group() {
      babelHelpers.classCallCheck(this, Group);
      return babelHelpers.possibleConstructorReturn(this, Object.getPrototypeOf(Group).apply(this, arguments));
    }

    babelHelpers.createClass(Group, [{
      key: 'render',
      value: function render() {
        var _props = this.props;
        var blocks = _props.blocks;
        var children = _props.children;
        var goTo = _props.goTo;
        var onClick = _props.onClick;
        var baseStyle = _props.style;
        var teleportTo = _props.teleportTo;
        var propsRenderBlocks = _props.renderBlocks;
        var rest = babelHelpers.objectWithoutProperties(_props, ['blocks', 'children', 'goTo', 'onClick', 'style', 'teleportTo', 'renderBlocks']);

        var renderBlocks = this.context.renderBlocks || propsRenderBlocks;

        var baseProps = {};
        var Base = void 0;

        if (teleportTo) {
          Base = panelsUi.Teleport;
          baseProps.to = teleportTo;
        } else if (goTo) {
          Base = GoTo;
          baseProps.href = goTo;
          baseProps.target = '_blank';
        } else if (onClick) {
          Base = OnClick;
          baseProps.onClick = onClick;
        } else {
          Base = 'div';
        }

        return React__default.createElement(
          Base,
          babelHelpers.extends({ style: babelHelpers.extends({}, baseStyle, style, browserVendorPrefix.flexWrapWrap) }, rest, baseProps),
          children || renderBlocks(blocks, name)
        );
      }
    }]);
    return Group;
  }(React.Component);

  Group.displayName = name;
  Group.contextTypes = {
    renderBlocks: React.PropTypes.func
  };
  return Group;
}

var _Horizontal = createGroup('Horizontal', browserVendorPrefix.flexDirectionRow);

var _Image = (function (props) {
  return React__default.createElement(
    'div',
    { style: props.styleWrapper },
    React__default.createElement('img', {
      alt: props.text,
      src: props.src,
      style: props.style })
  );
})

var _class = function (_Component) {
  babelHelpers.inherits(_class, _Component);

  function _class() {
    babelHelpers.classCallCheck(this, _class);
    return babelHelpers.possibleConstructorReturn(this, Object.getPrototypeOf(_class).apply(this, arguments));
  }

  babelHelpers.createClass(_class, [{
    key: 'render',
    value: function render() {
      var props = this.props;


      var backgroundColor = props.style && props.style.backgroundColor || 'transparent';
      var color = props.style && props.style.color || 'black';
      var className = 'Input-' + uniqueId();
      var inlineStyle = '.' + className + ':-webkit-autofill {\n      background-color: ' + backgroundColor + ' !important;\n      box-shadow: 0 0 0px 1000px ' + backgroundColor + ' inset;\n      color: ' + color + ' !important;\n    }';

      if (props.styleHover) {
        inlineStyle = inlineStyle + ' .' + className + ':hover {' + toCSS(props.styleHover) + '}';
      }

      return React__default.createElement(
        'div',
        { style: props.styleWrapper },
        React__default.createElement('input', {
          autoComplete: 'off',
          className: className,
          placeholder: props.placeholder,
          ref: 'input',
          style: props.style,
          type: props.type }),
        React__default.createElement(
          'style',
          null,
          inlineStyle
        )
      );
    }
  }]);
  return _class;
}(React.Component);

var Knocking = function Knocking(_ref) {
  var _ref$style = _ref.style;
  var style = _ref$style === undefined ? {} : _ref$style;
  var size = _ref.size;
  return React__default.createElement(
    'div',
    { style: style },
    React__default.createElement(Waiting, { color: style.color, size: size })
  );
};

var Launcher = function (_Component) {
  babelHelpers.inherits(Launcher, _Component);

  function Launcher(props) {
    babelHelpers.classCallCheck(this, Launcher);

    var _this = babelHelpers.possibleConstructorReturn(this, Object.getPrototypeOf(Launcher).call(this, props));

    _this.state = {};
    return _this;
  }

  babelHelpers.createClass(Launcher, [{
    key: 'onChange',
    value: function onChange(event) {
      this.setState({
        text: this.refs.text.value
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var text = this.state.text;
      var props = this.props;

      var _props$href$match = props.href.match(/(.*)TEXT(.*)/);

      var _props$href$match2 = babelHelpers.slicedToArray(_props$href$match, 3);

      var _ = _props$href$match2[0];
      var before = _props$href$match2[1];
      var after = _props$href$match2[2];

      var href = props.href.replace(/TEXT/g, text);

      return React__default.createElement(
        'div',
        { style: babelHelpers.extends({}, style$2.wrapper, props.style) },
        React__default.createElement(
          'div',
          { style: style$2.text },
          before,
          React__default.createElement('input', { type: props.input.type,
            onChange: function onChange(e) {
              return _this2.onChange(e);
            },
            placeholder: props.input.placeholder,
            ref: 'text',
            style: babelHelpers.extends({}, style$2.input, props.input.style) }),
          after
        ),
        text && React__default.createElement(
          panelsUi.Teleport,
          { href: href, style: props.action.style },
          props.action.text
        )
      );
    }
  }]);
  return Launcher;
}(React.Component);

Launcher.defaultProps = {
  input: {
    type: 'text'
  },
  action: {}
};
var style$2 = {
  input: {
    borderBottom: '1px solid black',
    fontSize: 14,
    outline: 0,
    paddingLeft: 5,
    paddingRight: 5
  },
  text: {
    flex: 1,
    flexDirection: 'row'
  },
  wrapper: {
    flexDirection: 'row',
    fontSize: 14
  }
};

function raw() {
  var props = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var superProps = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var rawProps = {};

  Object.keys(props).forEach(function (key) {
    var match = typeof props[key] === 'string' && props[key].match(/^item\.(.+)/);
    if (match) {
      rawProps[key] = superProps[match[1]];
    } else {
      if (key === 'blocks') {
        rawProps.blocks = props.blocks.map(function (block) {
          return morph(block, superProps);
        });
      } else if (key === 'props') {
        rawProps.props = raw(props.props, superProps);
      } else {
        rawProps[key] = props[key];
      }
    }
  });

  return rawProps;
}

function morph(block, item) {
  return babelHelpers.extends({}, block, {
    props: raw(block.props, item)
  });
}

var List = function (_Component) {
  babelHelpers.inherits(List, _Component);

  function List() {
    babelHelpers.classCallCheck(this, List);
    return babelHelpers.possibleConstructorReturn(this, Object.getPrototypeOf(List).apply(this, arguments));
  }

  babelHelpers.createClass(List, [{
    key: 'render',
    value: function render() {
      var props = this.props;

      var list = Array.isArray(props.list) ? props.list : [];
      var renderBlocks = this.context.renderBlocks || props.renderBlocks;

      return React__default.createElement(
        'div',
        null,
        renderBlocks(list.map(function (item) {
          return morph(props.block, item);
        }), 'list-' + uniqueId())
      );
    }
  }]);
  return List;
}(React.Component);

List.contextTypes = {
  renderBlocks: React.PropTypes.func
};

var Text = (function (_ref) {
  var _ref$element = _ref.element;
  var Element = _ref$element === undefined ? 'div' : _ref$element;
  var marginTop = _ref.lineBreak;
  var style = _ref.style;
  var _ref$text = _ref.text;
  var text = _ref$text === undefined ? '' : _ref$text;
  return React__default.createElement(
    Element,
    { style: style },
    ('' + text).split('\n').map(function (t, i) {
      return React__default.createElement(
        'div',
        { key: i, style: i ? { marginTop: marginTop } : {} },
        t
      );
    })
  );
})

var Preview = function Preview(_ref) {
  var Block = _ref.block;
  var _ref$props = _ref.props;
  var props = _ref$props === undefined ? {} : _ref$props;

  if (typeof Block === 'function') {
    return React__default.createElement(Block, props);
  } else {
    return React__default.createElement(
      'div',
      { style: styles.preview },
      React__default.createElement(Text, { style: styles.text, text: 'In an ideal world I\'d be rendering "' + Block + '" with these props:' }),
      React__default.createElement(Code, { code: JSON.stringify(props),
        gutter: false,
        highlightActiveLine: false,
        readOnly: true,
        theme: 'github',
        style: styles.code,
        wrap: 40 }),
      React__default.createElement(Text, { style: styles.text,
        text: 'However we got a ' + (typeof Block === 'undefined' ? 'undefined' : babelHelpers.typeof(Block)) + ' instead of a React element.\nThis is ok if you\'re using the Preview block in Pages to build Pages though :) (inception style :P). ¯\\_(ツ)_/¯' })
    );
  }
};
var styles = {
  code: {
    height: 100,
    marginTop: 20
  },
  text: {
    marginTop: 20
  },
  preview: {
    fontSize: 12,
    margin: 20
  }
};

var _Unknown = (function (props) {
  var _block = props._block;
  var rest = babelHelpers.objectWithoutProperties(props, ['_block']);


  var code = stringify({
    block: _block,
    props: rest
  }, { space: '\t' });

  return React__default.createElement(
    'div',
    { style: style$3.wrapper },
    React__default.createElement(
      'div',
      null,
      'Sorry but I don\'t know how to render this block :/'
    ),
    React__default.createElement(Code, { code: code, gutter: false, readOnly: true, highlightActiveLine: false, style: style$3.code })
  );
})

var style$3 = {
  code: {
    marginTop: 10
  },
  wrapper: {
    backgroundColor: '#323232',
    color: 'white',
    fontSize: 12,
    padding: 20
  }
};

var _Vertical = createGroup('Vertical', browserVendorPrefix.flexDirectionColumn);

var whitelist = ['autopause', 'autoplay', 'byline', 'color', 'height', 'loop', 'maxheight', 'maxwidth', 'portrait', 'title', 'width'];

function asParams(params) {
  return Object.keys(params).map(function (k) {
    return k + '=' + params[k];
  }).join('&');
}

function createSrc(props) {
  var video = props.video;
  var rest = babelHelpers.objectWithoutProperties(props, ['video']);


  var config = {
    autoplay: true,
    byline: false,
    title: false,
    url: 'https%3A//vimeo.com/' + video,
    width: 360
  };

  Object.keys(rest).forEach(function (key) {
    if (whitelist.indexOf(key) > -1) {
      config[key] = rest[key];
    }
  });

  return 'https://vimeo.com/api/oembed.json?' + asParams(config);
}

var _Vimeo = (function (props) {
  return React__default.createElement(Embed, babelHelpers.extends({ src: createSrc(props) }, props));
})

exports.Code = Code;
exports.Embed = Embed;
exports.Horizontal = _Horizontal;
exports.Image = _Image;
exports.Input = _class;
exports.Knocking = Knocking;
exports.Launcher = Launcher;
exports.List = List;
exports.Preview = Preview;
exports.Text = Text;
exports.Unknown = _Unknown;
exports.Vertical = _Vertical;
exports.Vimeo = _Vimeo;