import React, { Component, PropTypes } from 'react';
import uniqueId from 'mini-unique-id';
import Waiting from 'waiting';
import { flexDirectionRow, flexDirectionColumn, flexWrapWrap } from 'browser-vendor-prefix';
import toCSS from 'style-to-css';
import { Teleport } from 'panels-ui';
import stringify from 'json-stable-stringify';

var babelHelpers = {};

babelHelpers.extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

babelHelpers.objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

babelHelpers.slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

babelHelpers;

class Code extends Component {
  constructor(props) {
    super(props);
    this.id = `code-${ uniqueId() }`;
  }

  componentDidMount() {
    const props = this.props;

    const editor = ace.edit(this.id);
    this.editor = editor;

    const session = editor.getSession();

    session.setMode(`ace/mode/${ props.mode }`);
    session.setTabSize(2);
    session.setUseSoftTabs(true);
    session.setUseWrapMode(true);

    editor.setDisplayIndentGuides(false);
    editor.setFontSize(12);
    editor.setHighlightActiveLine(props.highlightActiveLine);
    editor.setTheme(`ace/theme/${ props.theme }`);
    editor.setOption('readOnly', props.readOnly);
    editor.setOption('wrap', props.wrap);
    editor.renderer.setShowGutter(props.gutter);

    if (props.src) {
      fetch(props.src).then(res => res.text()).then(code => this.editor.setValue(code, -1));
    } else if (props.code) {
      editor.setValue(props.code, -1);
    }
  }

  componentDidUpdate(prevProps) {
    const editor = this.editor;
    const props = this.props;

    const session = editor.getSession();

    if (prevProps.mode !== props.mode) {
      session.setMode(`ace/mode/${ props.mode }`);
    }
    if (prevProps.code !== props.code) {
      editor.setValue(props.code, -1);
    }
    if (prevProps.gutter !== props.gutter) {
      editor.renderer.setShowGutter(props.gutter);
    }
    if (prevProps.highlightActiveLine !== props.highlightActiveLine) {
      editor.setHighlightActiveLine(props.highlightActiveLine);
    }

    if (prevProps.readyOnly !== props.readOnly) {
      editor.setOption('readOnly', props.readOnly);
    }
    if (prevProps.src !== props.src) {
      fetch(props.src).then(res => res.text()).then(code => editor.setValue(code, -1));
    }
    if (prevProps.wrap !== props.wrap) {
      editor.setOption('wrap', props.wrap);
    }
  }

  render() {
    const id = this.id;


    return React.createElement(
      'div',
      { id: this.props.id },
      React.createElement(
        'style',
        null,
        `
          #${ id } span {
            display: inline;
          }
          #${ id } div {
            display: block;
          }
          #${ id } .ace_gutter-cell span {
            display: inline-block;
          }
        `
      ),
      React.createElement('div', { id: id, style: babelHelpers.extends({}, style, this.props.style) })
    );
  }
}

Code.defaultProps = {
  gutter: true,
  highlightActiveLine: true,
  mode: 'json',
  theme: 'idle_fingers',
  wrap: 40
};

const style = {
  height: 250,
  width: '100%'
};

const OverCover = props => {
  const overCoverStyle = babelHelpers.extends({}, style$1.overCover, {
    height: props.height,
    left: `calc(50% - ${ props.width / 2 }px)`,
    top: `calc(50% - ${ props.height / 2 }px)`,
    width: props.width
  });

  return React.createElement('img', { src: props.src, style: props.style });
};

class Embed extends Component {
  constructor(props) {
    super(props);

    this.state = {
      error: !!props.src === false,
      isLoading: !!props.src,
      isReady: false
    };
  }

  componentWillMount() {
    this.getEmbed(this.props.src);
  }

  componentWillReceiveProps(nextProps) {
    if (this.props.src !== nextProps.src) {
      this.getEmbed(nextProps.src);
    }
  }

  getEmbed(src) {
    fetch(src).then(res => res.json()).then(data => {
      this.setState({
        data,
        isLoading: false,
        isReady: true,
        showCover: !!data.thumbnail_url
      });
    }).catch(err => {
      this.setState({
        error: true,
        message: err
      });
    });
  }

  render() {
    var _state = this.state;
    const data = _state.data;
    const error = _state.error;
    const isLoading = _state.isLoading;
    const isReady = _state.isReady;
    const showCover = _state.showCover;
    var _props = this.props;
    const height = _props.height;
    const overCover = _props.overCover;
    const width = _props.width;

    const embedStyle = babelHelpers.extends({
      height,
      width
    }, this.props.style);
    let ret;

    if (isLoading) {
      ret = React.createElement(Waiting, null);
    } else if (isReady) {
      if (showCover) {
        ret = React.createElement(
          'div',
          { onClick: () => this.setState({ showCover: false }), style: style$1.cover },
          overCover && React.createElement(OverCover, overCover),
          React.createElement('img', { src: data.thumbnail_url, style: style$1.coverImage })
        );
      } else {
        ret = React.createElement('div', { dangerouslySetInnerHTML: { __html: data.html } });
      }
    } else if (error) {
      ret = React.createElement(
        'div',
        null,
        typeof message === 'undefined' ? 'Do you src in your props?' : message
      );
    }

    return React.createElement(
      'div',
      { style: embedStyle },
      ret
    );
  }
}
Embed.defaultProps = {
  overCover: false
};

const style$1 = {
  cover: {
    cursor: 'pointer',
    position: 'relative'
  },
  overCover: {
    position: 'absolute',
    zIndex: 1
  }
};

const GoTo = props => {
  const styleHover = props.styleHover;
  const rest = babelHelpers.objectWithoutProperties(props, ['styleHover']);

  const className = `GoTo-${ uniqueId() }`;

  const inlineStyle = props.styleHover ? `.${ className }:hover {${ toCSS(styleHover) }}` : '';

  return React.createElement(
    'a',
    babelHelpers.extends({}, rest, { className: className, target: '_blank' }),
    props.children,
    React.createElement(
      'style',
      null,
      inlineStyle
    )
  );
};

let clipboardText = null;

document.addEventListener('copy', ev => {
  if (clipboardText) {
    ev.preventDefault();
    ev.clipboardData.setData('text/plain', clipboardText);
    clipboardText = null;
  }
});

function copyToClipboard(text) {
  clipboardText = text;
  // debugger
  document.execCommand('copy');
}

const RESET_ACTIVE_TIMEOUT = 1000;

class OnClick extends Component {
  constructor(props) {
    super(props);

    this.state = {
      active: false
    };
  }

  bindOnClick(onClick) {
    let finalOnClick;

    if (/^copy:/.test(onClick)) {
      const from = onClick.match(/^copy:(.+)/)[1];

      finalOnClick = () => {
        const $el = document.querySelector(from);
        const $editor = Array.from($el.childNodes).find($c => $c.classList.contains('ace_editor'));
        const text = $editor ? ace.edit($editor.id).getValue() : $el.innerText;

        copyToClipboard(text);
      };
    } else if (typeof onClick === 'function') {
      finalOnClick = onClick;
    } else {
      finalOnClick = () => console.log(onClick);
    }

    this.onClick = event => {
      finalOnClick(event);

      this.setState({
        active: true
      });

      this.onClickTimeout = setTimeout(() => {
        this.setState({
          active: false
        });
        this.onClickTimeout = null;
      }, RESET_ACTIVE_TIMEOUT);
    };
  }

  componentWillMount() {
    this.bindOnClick(this.props.onClick);
  }

  componentWillUnmount() {
    if (this.onClickTimeout) {
      clearTimeout(this.onClickTimeout);
    }
  }

  componentWillUpdate(nextProps) {
    this.bindOnClick(nextProps.onClick);
  }

  render() {
    var _props = this.props;
    const children = _props.children;
    const onClick = _props.onClick;
    const style = _props.style;
    const styleActive = _props.styleActive;
    const styleHover = _props.styleHover;
    const rest = babelHelpers.objectWithoutProperties(_props, ['children', 'onClick', 'style', 'styleActive', 'styleHover']);
    const active = this.state.active;

    const className = `OnClick-${ uniqueId() }`;

    const inlineStyle = !active && styleHover ? `.${ className }:hover {${ toCSS(styleHover) }}` : '';

    const finalStyle = active ? babelHelpers.extends({}, style, styleActive, {
      outline: 0
    }) : babelHelpers.extends({}, style, {
      outline: 0,
      cursor: 'pointer'
    });

    return React.createElement(
      'button',
      babelHelpers.extends({}, rest, { className: className, disabled: active, onClick: () => this.onClick(), style: finalStyle }),
      children,
      React.createElement(
        'style',
        null,
        inlineStyle
      )
    );
  }
}

function createGroup(name, style) {
  class Group extends Component {
    render() {
      var _props = this.props;
      const blocks = _props.blocks;
      const children = _props.children;
      const goTo = _props.goTo;
      const onClick = _props.onClick;
      const baseStyle = _props.style;
      const teleportTo = _props.teleportTo;
      const propsRenderBlocks = _props.renderBlocks;
      const rest = babelHelpers.objectWithoutProperties(_props, ['blocks', 'children', 'goTo', 'onClick', 'style', 'teleportTo', 'renderBlocks']);

      const renderBlocks = this.context.renderBlocks || propsRenderBlocks;

      const baseProps = {};
      let Base;

      if (teleportTo) {
        Base = Teleport;
        baseProps.to = teleportTo;
      } else if (goTo) {
        Base = GoTo;
        baseProps.href = goTo;
        baseProps.target = '_blank';
      } else if (onClick) {
        Base = OnClick;
        baseProps.onClick = onClick;
      } else {
        Base = 'div';
      }

      return React.createElement(
        Base,
        babelHelpers.extends({ style: babelHelpers.extends({}, baseStyle, style, flexWrapWrap) }, rest, baseProps),
        children || renderBlocks(blocks, name)
      );
    }
  }
  Group.displayName = name;
  Group.contextTypes = {
    renderBlocks: PropTypes.func
  };
  return Group;
}

var _Horizontal = createGroup('Horizontal', flexDirectionRow);

var _Image = (props => React.createElement(
  'div',
  { style: props.styleWrapper },
  React.createElement('img', {
    alt: props.text,
    src: props.src,
    style: props.style })
))

var _Input = class extends Component {
  render() {
    const props = this.props;


    const backgroundColor = props.style && props.style.backgroundColor || 'transparent';
    const color = props.style && props.style.color || 'black';
    const className = `Input-${ uniqueId() }`;
    let inlineStyle = `.${ className }:-webkit-autofill {
      background-color: ${ backgroundColor } !important;
      box-shadow: 0 0 0px 1000px ${ backgroundColor } inset;
      color: ${ color } !important;
    }`;

    if (props.styleHover) {
      inlineStyle = `${ inlineStyle } .${ className }:hover {${ toCSS(props.styleHover) }}`;
    }

    return React.createElement(
      'div',
      { style: props.styleWrapper },
      React.createElement('input', {
        autoComplete: 'off',
        className: className,
        placeholder: props.placeholder,
        ref: 'input',
        style: props.style,
        type: props.type }),
      React.createElement(
        'style',
        null,
        inlineStyle
      )
    );
  }
}

const Knocking = ({ style = {}, size }) => React.createElement(
  'div',
  { style: style },
  React.createElement(Waiting, { color: style.color, size: size })
);

class Launcher extends Component {
  constructor(props) {
    super(props);

    this.state = {};
  }

  onChange(event) {
    this.setState({
      text: this.refs.text.value
    });
  }

  render() {
    const text = this.state.text;
    const props = this.props;

    var _props$href$match = props.href.match(/(.*)TEXT(.*)/);

    var _props$href$match2 = babelHelpers.slicedToArray(_props$href$match, 3);

    const _ = _props$href$match2[0];
    const before = _props$href$match2[1];
    const after = _props$href$match2[2];

    const href = props.href.replace(/TEXT/g, text);

    return React.createElement(
      'div',
      { style: babelHelpers.extends({}, style$2.wrapper, props.style) },
      React.createElement(
        'div',
        { style: style$2.text },
        before,
        React.createElement('input', { type: props.input.type,
          onChange: e => this.onChange(e),
          placeholder: props.input.placeholder,
          ref: 'text',
          style: babelHelpers.extends({}, style$2.input, props.input.style) }),
        after
      ),
      text && React.createElement(
        Teleport,
        { href: href, style: props.action.style },
        props.action.text
      )
    );
  }
}
Launcher.defaultProps = {
  input: {
    type: 'text'
  },
  action: {}
};
const style$2 = {
  input: {
    borderBottom: '1px solid black',
    fontSize: 14,
    outline: 0,
    paddingLeft: 5,
    paddingRight: 5
  },
  text: {
    flex: 1,
    flexDirection: 'row'
  },
  wrapper: {
    flexDirection: 'row',
    fontSize: 14
  }
};

function raw(props = {}, superProps = {}) {
  let rawProps = {};

  Object.keys(props).forEach(key => {
    const match = typeof props[key] === 'string' && props[key].match(/^item\.(.+)/);
    if (match) {
      rawProps[key] = superProps[match[1]];
    } else {
      if (key === 'blocks') {
        rawProps.blocks = props.blocks.map(block => morph(block, superProps));
      } else if (key === 'props') {
        rawProps.props = raw(props.props, superProps);
      } else {
        rawProps[key] = props[key];
      }
    }
  });

  return rawProps;
}

function morph(block, item) {
  return babelHelpers.extends({}, block, {
    props: raw(block.props, item)
  });
}

class List extends Component {
  render() {
    const props = this.props;

    const list = Array.isArray(props.list) ? props.list : [];
    const renderBlocks = this.context.renderBlocks || props.renderBlocks;

    return React.createElement(
      'div',
      null,
      renderBlocks(list.map(item => morph(props.block, item)), `list-${ uniqueId() }`)
    );
  }
}
List.contextTypes = {
  renderBlocks: PropTypes.func
};

var Text = (({ element: Element = 'div', lineBreak: marginTop, style, text = '' }) => React.createElement(
  Element,
  { style: style },
  `${ text }`.split('\n').map((t, i) => React.createElement(
    'div',
    { key: i, style: i ? { marginTop } : {} },
    t
  ))
))

const Preview = ({ block: Block, props = {} }) => {
  if (typeof Block === 'function') {
    return React.createElement(Block, props);
  } else {
    return React.createElement(
      'div',
      { style: styles.preview },
      React.createElement(Text, { style: styles.text, text: `In an ideal world I'd be rendering "${ Block }" with these props:` }),
      React.createElement(Code, { code: JSON.stringify(props),
        gutter: false,
        highlightActiveLine: false,
        readOnly: true,
        theme: 'github',
        style: styles.code,
        wrap: 40 }),
      React.createElement(Text, { style: styles.text,
        text: `However we got a ${ typeof Block } instead of a React element.\nThis is ok if you're using the Preview block in Pages to build Pages though :) (inception style :P). ¯\\_(ツ)_/¯` })
    );
  }
};
const styles = {
  code: {
    height: 100,
    marginTop: 20
  },
  text: {
    marginTop: 20
  },
  preview: {
    fontSize: 12,
    margin: 20
  }
};

var _Unknown = (props => {
  const _block = props._block;
  const rest = babelHelpers.objectWithoutProperties(props, ['_block']);


  const code = stringify({
    block: _block,
    props: rest
  }, { space: '\t' });

  return React.createElement(
    'div',
    { style: style$3.wrapper },
    React.createElement(
      'div',
      null,
      'Sorry but I don\'t know how to render this block :/'
    ),
    React.createElement(Code, { code: code, gutter: false, readOnly: true, highlightActiveLine: false, style: style$3.code })
  );
})

const style$3 = {
  code: {
    marginTop: 10
  },
  wrapper: {
    backgroundColor: '#323232',
    color: 'white',
    fontSize: 12,
    padding: 20
  }
};

var _Vertical = createGroup('Vertical', flexDirectionColumn);

const whitelist = ['autopause', 'autoplay', 'byline', 'color', 'height', 'loop', 'maxheight', 'maxwidth', 'portrait', 'title', 'width'];

function asParams(params) {
  return Object.keys(params).map(k => `${ k }=${ params[k] }`).join('&');
}

function createSrc(props) {
  const video = props.video;
  const rest = babelHelpers.objectWithoutProperties(props, ['video']);


  const config = {
    autoplay: true,
    byline: false,
    title: false,
    url: `https%3A//vimeo.com/${ video }`,
    width: 360
  };

  Object.keys(rest).forEach(key => {
    if (whitelist.indexOf(key) > -1) {
      config[key] = rest[key];
    }
  });

  return `https://vimeo.com/api/oembed.json?${ asParams(config) }`;
}

var _Vimeo = (props => React.createElement(Embed, babelHelpers.extends({ src: createSrc(props) }, props)))

export { Code, Embed, _Horizontal as Horizontal, _Image as Image, _Input as Input, _Knocking as Knocking, _Launcher as Launcher, _List as List, _Preview as Preview, Text, _Unknown as Unknown, _Vertical as Vertical, _Vimeo as Vimeo };